@@ -26,7 +26,6 @@ import sys
 import time
 import cPickle as pickle
 import errno
-from random import randint
 from tempfile import mkstemp
 
 from eventlet import sleep, Timeout
@@ -275,7 +274,7 @@ class DatabaseBroker(object):
         elif 'file is encrypted or is not a database' in str(exc_value):
             exc_hint = 'corrupted'
         else:
-            raise exc_type, exc_value, exc_traceback
+            raise exc_type(*exc_value.args), None, exc_traceback
         prefix_path = os.path.dirname(self.db_dir)
         partition_path = os.path.dirname(prefix_path)
         dbs_path = os.path.dirname(partition_path)
@@ -311,7 +310,7 @@ class DatabaseBroker(object):
             yield conn
             conn.rollback()
             self.conn = conn
-        except sqlite3.DatabaseError, err:
+        except sqlite3.DatabaseError:
             try:
                 conn.close()
             except:
@@ -1443,21 +1442,28 @@ class AccountBroker(DatabaseBroker):
         if self.db_file == ':memory:':
             self.merge_items([record])
             return
-        commit = False
-        with lock_parent_directory(self.pending_file, self.pending_timeout):
-            with open(self.pending_file, 'a+b') as fp:
-                # Colons aren't used in base64 encoding; so they are our
-                # delimiter
-                fp.write(':')
-                fp.write(pickle.dumps(
-                    (name, put_timestamp, delete_timestamp, object_count,
-                     bytes_used, deleted),
-                    protocol=PICKLE_PROTOCOL).encode('base64'))
-                fp.flush()
-                if fp.tell() > PENDING_CAP:
-                    commit = True
-        if commit:
-            self._commit_puts()
+        if not os.path.exists(self.db_file):
+            raise DatabaseConnectionError(self.db_file, "DB doesn't exist")
+        pending_size = 0
+        try:
+            pending_size = os.path.getsize(self.pending_file)
+        except OSError, err:
+            if err.errno != errno.ENOENT:
+                raise
+        if pending_size > PENDING_CAP:
+            self._commit_puts([record])
+        else:
+            with lock_parent_directory(self.pending_file,
+                                       self.pending_timeout):
+                with open(self.pending_file, 'a+b') as fp:
+                    # Colons aren't used in base64 encoding; so they are our
+                    # delimiter
+                    fp.write(':')
+                    fp.write(pickle.dumps(
+                        (name, put_timestamp, delete_timestamp, object_count,
+                         bytes_used, deleted),
+                        protocol=PICKLE_PROTOCOL).encode('base64'))
+                    fp.flush()
 
     def can_delete_db(self, cutoff):
         """
