@@ -240,7 +240,7 @@ class BucketController(Controller):
             if status == 401:
                 return get_err_response('AccessDenied')
             elif status == 404:
-                return get_err_response('InvalidBucketName')
+                return get_err_response('NoSuchBucket')
             else:
                 return get_err_response('InvalidURI')
 
@@ -267,7 +267,7 @@ class BucketController(Controller):
                 'true' if len(objects) == (max_keys + 1) else 'false',
                 max_keys,
                 xml_escape(self.container_name),
-                "".join(['<Contents><Key>%s</Key><LastModified>%s</LastModif'\
+                "".join(['<Contents><Key>%s</Key><LastModified>%sZ</LastModif'\
                         'ied><ETag>%s</ETag><Size>%s</Size><StorageClass>STA'\
                         'NDARD</StorageClass></Contents>' %
                         (xml_escape(i['name']), i['last_modified'], i['hash'],
@@ -311,7 +311,7 @@ class BucketController(Controller):
             if status == 401:
                 return get_err_response('AccessDenied')
             elif status == 404:
-                return get_err_response('InvalidBucketName')
+                return get_err_response('NoSuchBucket')
             elif status == 409:
                 return get_err_response('BucketNotEmpty')
             else:
@@ -397,7 +397,7 @@ class ObjectController(Controller):
             if status == 401:
                 return get_err_response('AccessDenied')
             elif status == 404:
-                return get_err_response('InvalidBucketName')
+                return get_err_response('NoSuchBucket')
             else:
                 return get_err_response('InvalidURI')
 
