@@ -850,6 +850,11 @@ log_name = %(yarr)s'''
         for v in utils.TRUE_VALUES:
             self.assertEquals(v, v.lower())
 
+    def test_streq_const_time(self):
+        self.assertTrue(utils.streq_const_time('abc123', 'abc123'))
+        self.assertFalse(utils.streq_const_time('a', 'aaaaa'))
+        self.assertFalse(utils.streq_const_time('ABC123', 'abc123'))
+
 
 if __name__ == '__main__':
     unittest.main()
