@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,8 +22,7 @@ from shutil import rmtree
 from time import time
 from distutils.dir_util import mkpath
 
-from eventlet import spawn, TimeoutError, listen
-from eventlet.timeout import Timeout
+from eventlet import spawn, Timeout, listen
 
 from swift.obj import updater as object_updater, server as object_server
 from swift.obj.server import ASYNCDIR
